# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ..account_base.types.account_id import AccountId
from ..common.errors.bad_request_error import BadRequestError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.types.id import Id
from ..common.types.problem import Problem
from ..integration_base.types.integration_id import IntegrationId
from ..token_base.types.token_id import TokenId
from .types.create_integration_token_request import CreateIntegrationTokenRequest
from .types.create_integration_token_response import CreateIntegrationTokenResponse
from .types.create_synqly_integrations_token_request import CreateSynqlyIntegrationsTokenRequest
from .types.create_synqly_integrations_token_response import CreateSynqlyIntegrationsTokenResponse
from .types.create_token_request import CreateTokenRequest
from .types.create_token_response import CreateTokenResponse
from .types.get_token_response import GetTokenResponse
from .types.list_tokens_response import ListTokensResponse
from .types.refresh_token_response import RefreshTokenResponse
from .types.reset_token_response import ResetTokenResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TokensClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_token(
        self, *, request: CreateTokenRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateTokenResponse:
        """
        Create an adhoc organization token restricted to specified resources and permission set.
        Tokens can only be reduced in scope, never expanded.
        Permissions are inherited from the token used to call this API.
        Permissions assigned to the new token will not be persisted, this is not a way to create roles.

        Parameters:
            - request: CreateTokenRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/tokens"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_integration_token(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        request: CreateIntegrationTokenRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateIntegrationTokenResponse:
        """
        Create an adhoc integration token restricted to a single integration. The token used to call
        this API must have the necessary permissions to create tokens and have access to the account
        and integration IDs. Permissions may not be escalated, so any operation that the invocation
        token does not have access to cannot be granted.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - request: CreateIntegrationTokenRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/tokens/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateIntegrationTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_synqly_integrations_token(
        self, *, request: CreateSynqlyIntegrationsTokenRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateSynqlyIntegrationsTokenResponse:
        """
        Create a token for managing Synqly-specific integrations. This token can be used with the integration APIs to manage
        integrations for Synqly-specific integrations, such as status events exports and async operations. See the
        [Synqly Integrations](https://docs.synqly.com/guides/synqly-integrations) documentation for more information.

        Parameters:
            - request: CreateSynqlyIntegrationsTokenRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/tokens/synqly-integrations"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateSynqlyIntegrationsTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes the Refresh Token with id `{id}`. This immediately
        invalidates both the primary and secondary token pairs.

        Parameters:
            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/tokens/{jsonable_encoder(refresh_token_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListTokensResponse:
        """
        Returns a list of all `RefreshToken` objects belonging to the Authorization Bearer
        token. For more infromation on Tokens, refer to
        [Authentication](/api-reference/authentication).

        Parameters:
            - limit: typing.Optional[int]. Number of `Token` objects to return in this page. Defaults to 100.

            - start_after: typing.Optional[str]. Return `Token` objects starting after this `name`.

            - order: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Select a field to order the results by. Defaults to `name`. To control the direction of the sorting, append
                                                                               `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order.
                                                                               The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
                                                                               ordering is applied in the order the fields are specified.
            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
                                                                                If used more than once, the queries are ANDed together.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/tokens"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "start_after": start_after,
                        "order": order,
                        "filter": filter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListTokensResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTokenResponse:
        """
        Returns the `RefreshToken` object matching `{tokenId}`. For more information on
        Tokens, refer to
        [Authentication](/api-reference/authentication).

        Parameters:
            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/tokens/{jsonable_encoder(refresh_token_id)}/info"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def reset(
        self, owner_id: Id, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ResetTokenResponse:
        """
        This API can be used to reset `Organization` or `Integration` `RefreshTokens`.
        Resets the specified `RefreshToken` and expiration time, removes the secondary, and resets access and refresh tokens for the
        `RefreshToken` object matching `{ownerId}/{refreshTokenId}` where `ownerId` is an `organizationId` or `integrationId`.
        An `Organization` token with `administrator` permissions can be used to perform this operation.

        Parameters:
            - owner_id: Id.

            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/tokens/{jsonable_encoder(owner_id)}/{jsonable_encoder(refresh_token_id)}/reset",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResetTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def refresh(
        self, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RefreshTokenResponse:
        """
        Creates a new primary `TokenPair` object, setting the secondary `TokenPair`
        to the previous primary value. Call `/v1/removeSecondaryToken` to remove
        this secondary backup once the new primary `TokenPair` has been deployed.

        Parameters:
            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/tokens/{jsonable_encoder(refresh_token_id)}/refresh"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RefreshTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def remove_secondary(
        self, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes the secondary `TokenPair` for the `RefreshToken` object
        matching `{refreshTokenId}`.

        Parameters:
            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/tokens/{jsonable_encoder(refresh_token_id)}/secondary"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTokensClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_token(
        self, *, request: CreateTokenRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateTokenResponse:
        """
        Create an adhoc organization token restricted to specified resources and permission set.
        Tokens can only be reduced in scope, never expanded.
        Permissions are inherited from the token used to call this API.
        Permissions assigned to the new token will not be persisted, this is not a way to create roles.

        Parameters:
            - request: CreateTokenRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/tokens"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_integration_token(
        self,
        account_id: AccountId,
        integration_id: IntegrationId,
        *,
        request: CreateIntegrationTokenRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateIntegrationTokenResponse:
        """
        Create an adhoc integration token restricted to a single integration. The token used to call
        this API must have the necessary permissions to create tokens and have access to the account
        and integration IDs. Permissions may not be escalated, so any operation that the invocation
        token does not have access to cannot be granted.

        Parameters:
            - account_id: AccountId.

            - integration_id: IntegrationId.

            - request: CreateIntegrationTokenRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/tokens/{jsonable_encoder(account_id)}/{jsonable_encoder(integration_id)}",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateIntegrationTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_synqly_integrations_token(
        self, *, request: CreateSynqlyIntegrationsTokenRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateSynqlyIntegrationsTokenResponse:
        """
        Create a token for managing Synqly-specific integrations. This token can be used with the integration APIs to manage
        integrations for Synqly-specific integrations, such as status events exports and async operations. See the
        [Synqly Integrations](https://docs.synqly.com/guides/synqly-integrations) documentation for more information.

        Parameters:
            - request: CreateSynqlyIntegrationsTokenRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/tokens/synqly-integrations"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateSynqlyIntegrationsTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes the Refresh Token with id `{id}`. This immediately
        invalidates both the primary and secondary token pairs.

        Parameters:
            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/tokens/{jsonable_encoder(refresh_token_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListTokensResponse:
        """
        Returns a list of all `RefreshToken` objects belonging to the Authorization Bearer
        token. For more infromation on Tokens, refer to
        [Authentication](/api-reference/authentication).

        Parameters:
            - limit: typing.Optional[int]. Number of `Token` objects to return in this page. Defaults to 100.

            - start_after: typing.Optional[str]. Return `Token` objects starting after this `name`.

            - order: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Select a field to order the results by. Defaults to `name`. To control the direction of the sorting, append
                                                                               `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order.
                                                                               The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
                                                                               ordering is applied in the order the fields are specified.
            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
                                                                                If used more than once, the queries are ANDed together.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/tokens"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "limit": limit,
                        "start_after": start_after,
                        "order": order,
                        "filter": filter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListTokensResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTokenResponse:
        """
        Returns the `RefreshToken` object matching `{tokenId}`. For more information on
        Tokens, refer to
        [Authentication](/api-reference/authentication).

        Parameters:
            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/tokens/{jsonable_encoder(refresh_token_id)}/info"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def reset(
        self, owner_id: Id, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ResetTokenResponse:
        """
        This API can be used to reset `Organization` or `Integration` `RefreshTokens`.
        Resets the specified `RefreshToken` and expiration time, removes the secondary, and resets access and refresh tokens for the
        `RefreshToken` object matching `{ownerId}/{refreshTokenId}` where `ownerId` is an `organizationId` or `integrationId`.
        An `Organization` token with `administrator` permissions can be used to perform this operation.

        Parameters:
            - owner_id: Id.

            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/tokens/{jsonable_encoder(owner_id)}/{jsonable_encoder(refresh_token_id)}/reset",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ResetTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def refresh(
        self, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RefreshTokenResponse:
        """
        Creates a new primary `TokenPair` object, setting the secondary `TokenPair`
        to the previous primary value. Call `/v1/removeSecondaryToken` to remove
        this secondary backup once the new primary `TokenPair` has been deployed.

        Parameters:
            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/tokens/{jsonable_encoder(refresh_token_id)}/refresh"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RefreshTokenResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def remove_secondary(
        self, refresh_token_id: TokenId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes the secondary `TokenPair` for the `RefreshToken` object
        matching `{refreshTokenId}`.

        Parameters:
            - refresh_token_id: TokenId.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/tokens/{jsonable_encoder(refresh_token_id)}/secondary"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ListIntegrationOptions(str, enum.Enum):
    ACCOUNT = "account"
    ACCOUNTS = "accounts"
    INTEGRATION_POINT = "integration_point"
    INTEGRATION_POINTS = "integration_points"
    ALL = "all"

    def visit(
        self,
        account: typing.Callable[[], T_Result],
        accounts: typing.Callable[[], T_Result],
        integration_point: typing.Callable[[], T_Result],
        integration_points: typing.Callable[[], T_Result],
        all_: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ListIntegrationOptions.ACCOUNT:
            return account()
        if self is ListIntegrationOptions.ACCOUNTS:
            return accounts()
        if self is ListIntegrationOptions.INTEGRATION_POINT:
            return integration_point()
        if self is ListIntegrationOptions.INTEGRATION_POINTS:
            return integration_points()
        if self is ListIntegrationOptions.ALL:
            return all_()



# This file was auto-generated by Fern from our API Definition.

from .activity_id import ActivityId
from .category_uid import CategoryUid
from .class_uid import ClassUid
from .severity_id import SeverityId
from .software_info import SoftwareInfo
from .status_id import StatusId
from .type_uid import TypeUid

__all__ = ["ActivityId", "CategoryUid", "ClassUid", "SeverityId", "SoftwareInfo", "StatusId", "TypeUid"]



# This file was auto-generated by Fern from our API Definition.

from .....v_1_3_0.resources.entitymanagement.resources.classes.types.entity_management import (
    EntityManagement as resources_ocsf_resources_v_1_3_0_resources_entitymanagement_resources_classes_types_entity_management_EntityManagement,
)

EntityManagement = resources_ocsf_resources_v_1_3_0_resources_entitymanagement_resources_classes_types_entity_management_EntityManagement



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ResourceId(str, enum.Enum):
    ALERTS = "alerts"
    APPLICATIONS = "applications"
    AUDIT_LOGS = "audit_logs"
    COMMENTS = "comments"
    DEVICES = "devices"
    EVENTS = "events"
    EVIDENCE = "evidence"
    FINDINGS = "findings"
    GROUPS = "groups"
    INVESTIGATIONS = "investigations"
    IOCS = "iocs"
    LOG_PROVIDERS = "log_providers"
    POSTURE_SCORES = "posture_scores"
    PROJECTS = "projects"
    SCANS = "scans"
    SCAN_ACTIVITIES = "scan_activities"
    THREATS = "threats"
    TICKETS = "tickets"
    USERS = "users"
    COMPLIANCE = "compliance"
    CLOUDRESOURCEINVENTORY = "cloudresourceinventory"

    def visit(
        self,
        alerts: typing.Callable[[], T_Result],
        applications: typing.Callable[[], T_Result],
        audit_logs: typing.Callable[[], T_Result],
        comments: typing.Callable[[], T_Result],
        devices: typing.Callable[[], T_Result],
        events: typing.Callable[[], T_Result],
        evidence: typing.Callable[[], T_Result],
        findings: typing.Callable[[], T_Result],
        groups: typing.Callable[[], T_Result],
        investigations: typing.Callable[[], T_Result],
        iocs: typing.Callable[[], T_Result],
        log_providers: typing.Callable[[], T_Result],
        posture_scores: typing.Callable[[], T_Result],
        projects: typing.Callable[[], T_Result],
        scans: typing.Callable[[], T_Result],
        scan_activities: typing.Callable[[], T_Result],
        threats: typing.Callable[[], T_Result],
        tickets: typing.Callable[[], T_Result],
        users: typing.Callable[[], T_Result],
        compliance: typing.Callable[[], T_Result],
        cloudresourceinventory: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ResourceId.ALERTS:
            return alerts()
        if self is ResourceId.APPLICATIONS:
            return applications()
        if self is ResourceId.AUDIT_LOGS:
            return audit_logs()
        if self is ResourceId.COMMENTS:
            return comments()
        if self is ResourceId.DEVICES:
            return devices()
        if self is ResourceId.EVENTS:
            return events()
        if self is ResourceId.EVIDENCE:
            return evidence()
        if self is ResourceId.FINDINGS:
            return findings()
        if self is ResourceId.GROUPS:
            return groups()
        if self is ResourceId.INVESTIGATIONS:
            return investigations()
        if self is ResourceId.IOCS:
            return iocs()
        if self is ResourceId.LOG_PROVIDERS:
            return log_providers()
        if self is ResourceId.POSTURE_SCORES:
            return posture_scores()
        if self is ResourceId.PROJECTS:
            return projects()
        if self is ResourceId.SCANS:
            return scans()
        if self is ResourceId.SCAN_ACTIVITIES:
            return scan_activities()
        if self is ResourceId.THREATS:
            return threats()
        if self is ResourceId.TICKETS:
            return tickets()
        if self is ResourceId.USERS:
            return users()
        if self is ResourceId.COMPLIANCE:
            return compliance()
        if self is ResourceId.CLOUDRESOURCEINVENTORY:
            return cloudresourceinventory()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class MappingsActions(str, enum.Enum):
    LIST = "list"
    CREATE = "create"
    GET = "get"
    UPDATE = "update"
    PATCH = "patch"
    DELETE = "delete"
    APPLY = "apply"
    ALL = "*"

    def visit(
        self,
        list_: typing.Callable[[], T_Result],
        create: typing.Callable[[], T_Result],
        get: typing.Callable[[], T_Result],
        update: typing.Callable[[], T_Result],
        patch: typing.Callable[[], T_Result],
        delete: typing.Callable[[], T_Result],
        apply: typing.Callable[[], T_Result],
        all_: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is MappingsActions.LIST:
            return list_()
        if self is MappingsActions.CREATE:
            return create()
        if self is MappingsActions.GET:
            return get()
        if self is MappingsActions.UPDATE:
            return update()
        if self is MappingsActions.PATCH:
            return patch()
        if self is MappingsActions.DELETE:
            return delete()
        if self is MappingsActions.APPLY:
            return apply()
        if self is MappingsActions.ALL:
            return all_()



# This file was auto-generated by Fern from our API Definition.

from .types import (
    BuiltinRoles,
    CreateRoleRequest,
    CreateRoleResponse,
    GetRoleResponse,
    ListRolesResponse,
    PatchRoleResponse,
    RoleDefinition,
    UpdateRoleRequest,
    UpdateRoleResponse,
)

__all__ = [
    "BuiltinRoles",
    "CreateRoleRequest",
    "CreateRoleResponse",
    "GetRoleResponse",
    "ListRolesResponse",
    "PatchRoleResponse",
    "RoleDefinition",
    "UpdateRoleRequest",
    "UpdateRoleResponse",
]



# This file was auto-generated by Fern from our API Definition.

from .apply_mapping_request import ApplyMappingRequest
from .apply_mapping_response import ApplyMappingResponse
from .apply_mapping_response_result import ApplyMappingResponseResult
from .create_mapping_request import CreateMappingRequest
from .create_mapping_response import CreateMappingResponse
from .create_mapping_response_result import CreateMappingResponseResult
from .get_mapping_response import GetMappingResponse
from .list_mappings_response import ListMappingsResponse
from .mapping import Mapping
from .mapping_base import MappingBase
from .mapping_id import MappingId
from .patch_mapping_response import PatchMappingResponse
from .update_mapping_request import UpdateMappingRequest
from .update_mapping_response import UpdateMappingResponse

__all__ = [
    "ApplyMappingRequest",
    "ApplyMappingResponse",
    "ApplyMappingResponseResult",
    "CreateMappingRequest",
    "CreateMappingResponse",
    "CreateMappingResponseResult",
    "GetMappingResponse",
    "ListMappingsResponse",
    "Mapping",
    "MappingBase",
    "MappingId",
    "PatchMappingResponse",
    "UpdateMappingRequest",
    "UpdateMappingResponse",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Application,
    ConnectionState,
    CreateIocsRequest,
    CreateIocsResponse,
    DeleteIocsResponse,
    GetEndpointResponse,
    NetworkQuarantineRequest,
    NetworkQuarantineResponse,
    PostureScore,
    QueryAlertsResponse,
    QueryApplicationsResponse,
    QueryEdrEventsResponse,
    QueryEndpointsResponse,
    QueryIocsResponse,
    QueryPostureScoreResponse,
    QueryThreatsResponse,
    ThreatEvent,
)

__all__ = [
    "Application",
    "ConnectionState",
    "CreateIocsRequest",
    "CreateIocsResponse",
    "DeleteIocsResponse",
    "GetEndpointResponse",
    "NetworkQuarantineRequest",
    "NetworkQuarantineResponse",
    "PostureScore",
    "QueryAlertsResponse",
    "QueryApplicationsResponse",
    "QueryEdrEventsResponse",
    "QueryEndpointsResponse",
    "QueryIocsResponse",
    "QueryPostureScoreResponse",
    "QueryThreatsResponse",
    "ThreatEvent",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class IndicatorTypeOv(str, enum.Enum):
    ANOMALOUS_ACTIVITY = "anomalous_activity"
    ANONYMIZATION = "anonymization"
    BENIGN = "benign"
    COMPROMISED = "compromised"
    MALICIOUS_ACTIVITY = "malicious_activity"
    ATTRIBUTION = "attribution"
    UNKNOWN = "unknown"

    def visit(
        self,
        anomalous_activity: typing.Callable[[], T_Result],
        anonymization: typing.Callable[[], T_Result],
        benign: typing.Callable[[], T_Result],
        compromised: typing.Callable[[], T_Result],
        malicious_activity: typing.Callable[[], T_Result],
        attribution: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is IndicatorTypeOv.ANOMALOUS_ACTIVITY:
            return anomalous_activity()
        if self is IndicatorTypeOv.ANONYMIZATION:
            return anonymization()
        if self is IndicatorTypeOv.BENIGN:
            return benign()
        if self is IndicatorTypeOv.COMPROMISED:
            return compromised()
        if self is IndicatorTypeOv.MALICIOUS_ACTIVITY:
            return malicious_activity()
        if self is IndicatorTypeOv.ATTRIBUTION:
            return attribution()
        if self is IndicatorTypeOv.UNKNOWN:
            return unknown()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .webhook_item import WebhookItem

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class WebhookConfig(pydantic.BaseModel):
    provider_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    The key used by Synqly to verify incoming webhook payloads sent by the Provider. The format and requirements for this key is Provider specific:
    
    - **ServiceNow**: ServiceNow does not support webhook signing at this time.
    - **Jira**: Synqly does _not_ automatically configure Jira webhooks. A user with administrator privileges must configure the webhook, including the signing key. If this key is not specified, Synqly will _not_ validate incoming webhooks from Jira. It is strongly recommended that a key is specified and configured with Jira to ensure the integrity of incoming payloads.
    """

    items: typing.List[WebhookItem] = pydantic.Field()
    """
    List of webhooks for an integration. If the provider supports webhooks, they will be sent to the servers provided in this list.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.timestamp import Timestamp
from .database_type_id import DatabaseTypeId
from .group import Group

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Database(pydantic.BaseModel):
    """
    The database object is used for databases which are typically datastore services that contain an organized collection of structured and unstructured data or a types of data.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the database was known to have been created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the database was known to have been created.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the database.
    """

    groups: typing.Optional[typing.List[Group]] = pydantic.Field(default=None)
    """
    The group names to which the database belongs.
    """

    modified_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The most recent time when any changes, updates, or modifications were made within the database.
    """

    modified_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The most recent time when any changes, updates, or modifications were made within the database.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The database name, ordinarily as assigned by a database administrator.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size of the database in bytes.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The database type.
    """

    type_id: DatabaseTypeId = pydantic.Field()
    """
    The normalized identifier of the database type.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the database.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Environment(str, enum.Enum):
    TEST = "test"
    PROD = "prod"

    def visit(self, test: typing.Callable[[], T_Result], prod: typing.Callable[[], T_Result]) -> T_Result:
        if self is Environment.TEST:
            return test()
        if self is Environment.PROD:
            return prod()



# This file was auto-generated by Fern from our API Definition.

from .bridge_selector import BridgeSelector, BridgeSelector_Id, BridgeSelector_Labels
from .create_integration_request import CreateIntegrationRequest
from .create_integration_response import CreateIntegrationResponse
from .create_integration_response_result import CreateIntegrationResponseResult
from .get_integration_response import GetIntegrationResponse
from .integration import Integration
from .list_account_integrations_response import ListAccountIntegrationsResponse
from .list_integration_options import ListIntegrationOptions
from .list_integrations_response import ListIntegrationsResponse
from .patch_integration_response import PatchIntegrationResponse
from .update_integration_request import UpdateIntegrationRequest
from .update_integration_response import UpdateIntegrationResponse
from .verify_integration_request import VerifyIntegrationRequest
from .webhook_config import WebhookConfig
from .webhook_event import WebhookEvent
from .webhook_item import WebhookItem

__all__ = [
    "BridgeSelector",
    "BridgeSelector_Id",
    "BridgeSelector_Labels",
    "CreateIntegrationRequest",
    "CreateIntegrationResponse",
    "CreateIntegrationResponseResult",
    "GetIntegrationResponse",
    "Integration",
    "ListAccountIntegrationsResponse",
    "ListIntegrationOptions",
    "ListIntegrationsResponse",
    "PatchIntegrationResponse",
    "UpdateIntegrationRequest",
    "UpdateIntegrationResponse",
    "VerifyIntegrationRequest",
    "WebhookConfig",
    "WebhookEvent",
    "WebhookItem",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .problem_cause import ProblemCause
from .problem_details import ProblemDetails
from .problem_type import ProblemType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Problem(ProblemDetails):
    """
    Generic base type for problems in Synqly. This object carries both machine-readable data as well as information intended for display purposes.
    """

    type: typing.Optional[ProblemType] = None
    occurred_at: dt.datetime = pydantic.Field()
    """
    The date and time the problem occurred.
    """

    status: int = pydantic.Field()
    """
    The HTTP status code of the problem. Matches the HTTP response code sent by the server.
    """

    instance: str = pydantic.Field()
    """
    A URI reference that identifies the specific occurrence of the problem. It may or may not yield further information if dereferenced.
    """

    cause: typing.Optional[typing.List[ProblemCause]] = pydantic.Field(default=None)
    """
    A list of the root cause(s) for this problem occurrence. Includes at minimum one root cause, and is otherwise an unordered list of causes.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .....v_1_3_0.resources.securityfinding.resources.classes.types.security_finding import (
    SecurityFinding as resources_ocsf_resources_v_1_3_0_resources_securityfinding_resources_classes_types_security_finding_SecurityFinding,
)

SecurityFinding = (
    resources_ocsf_resources_v_1_3_0_resources_securityfinding_resources_classes_types_security_finding_SecurityFinding
)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ProviderConfigId(str, enum.Enum):
    """
    List of supported providers.
    """

    ASSETS_ARMIS_CENTRIX = "assets_armis_centrix"
    """
    Armis Centrix™ for Asset Management and Security
    """

    ASSETS_ARMIS_CENTRIX_MOCK = "assets_armis_centrix_mock"
    """
    [MOCK] Armis Centrix™ for Asset Management and Security
    """

    ASSETS_AXONIUS = "assets_axonius"
    """
    Axonius Asset Cloud
    """

    ASSETS_CROWD_STRIKE = "assets_crowdstrike"
    """
    CrowdStrike Falcon Spotlight
    """

    ASSETS_NOZOMI_VANTAGE = "assets_nozomi_vantage"
    """
    Nozomi Vantage
    """

    ASSETS_NOZOMI_VANTAGE_MOCK = "assets_nozomi_vantage_mock"
    """
    [MOCK] Nozomi Vantage
    """

    ASSETS_QUALYS_CLOUD = "assets_qualys_cloud"
    """
    Qualys Vulnerability Management, Detection & Response (VMDR)
    """

    ASSETS_QUALYS_CLOUD_MOCK = "assets_qualys_cloud_mock"
    """
    [MOCK] Qualys Vulnerability Management, Detection & Response (VMDR)
    """

    ASSETS_SERVICE_NOW = "assets_servicenow"
    """
    ServiceNow Configuration Management Database (CMDB)
    """

    ASSETS_SERVICE_NOW_MOCK = "assets_servicenow_mock"
    """
    [MOCK] ServiceNow Configuration Management Database (CMDB)
    """

    ASSETS_SEVCO = "assets_sevco"
    """
    Sevco for Asset Management and Security
    """

    ASSETS_TANIUM_CLOUD = "assets_tanium_cloud"
    """
    Tanium Vulnerability Management
    """

    ASSETS_TANIUM_CLOUD_MOCK = "assets_tanium_cloud_mock"
    """
    [MOCK] Tanium Vulnerability Management
    """

    CLOUD_SECURITY_CROWD_STRIKE = "cloudsecurity_crowdstrike"
    """
    CrowdStrike Falcon® Insight EDR
    """

    CLOUD_SECURITY_DEFENDER = "cloudsecurity_defender"
    """
    Microsoft Defender for Cloud
    """

    EDR_CROWD_STRIKE = "edr_crowdstrike"
    """
    CrowdStrike Falcon® Insight EDR
    """

    EDR_DEFENDER = "edr_defender"
    """
    Microsoft Defender for Endpoint
    """

    EDR_MALWAREBYTES = "edr_malwarebytes"
    """
    ThreatDown Endpoint Detection & Response
    """

    EDR_SENTINEL_ONE = "edr_sentinelone"
    """
    SentinelOne Singularity™ Endpoint
    """

    EDR_SOPHOS = "edr_sophos"
    """
    Sophos Endpoint
    """

    EDR_TANIUM = "edr_tanium"
    """
    Tanium EDR
    """

    IDENTITY_ENTRA_ID = "identity_entra_id"
    """
    Microsoft Entra ID
    """

    IDENTITY_GOOGLE = "identity_google"
    """
    Google Workspace
    """

    IDENTITY_OKTA = "identity_okta"
    """
    Okta Identity
    """

    IDENTITY_PING_ONE = "identity_pingone"
    """
    PingOne Cloud Platform
    """

    NOTIFICATIONS_JIRA = "notifications_jira"
    """
    Atlassian Jira
    """

    NOTIFICATIONS_MOCK = "notifications_mock_notifications"
    """
    Synqly Test Provider
    """

    NOTIFICATIONS_SLACK_WEBHOOK = "notifications_slack_webhook"
    """
    Slack Incoming Webhook
    """

    NOTIFICATIONS_SLACK = "notifications_slack"
    """
    Slack
    """

    NOTIFICATIONS_TEAMS = "notifications_teams"
    """
    Microsoft Teams
    """

    SIEM_CROWDSTRIKE = "siem_crowdstrike"
    """
    CrowdStrike Falcon® Next-Gen SIEM
    """

    SIEM_ELASTICSEARCH = "siem_elasticsearch"
    """
    Elastic SIEM
    """

    SIEM_GOOGLE_CHRONICLE = "siem_google_chronicle"
    """
    Google Security Operations (Chronicle Compatibility)
    """

    SIEM_GOOGLE_SECURITY_OPERATIONS = "siem_google_security_operations"
    """
    Google Security Operations
    """

    SIEM_MOCK = "siem_mock_siem"
    """
    Synqly Test Provider
    """

    SIEM_OPEN_SEARCH = "siem_opensearch"
    """
    OpenSearch SIEM
    """

    SIEM_Q_RADAR = "siem_q_radar"
    """
    IBM QRadar SIEM
    """

    SIEM_RAPID_7_INSIGHT_IDR = "siem_rapid7_insightidr"
    """
    Rapid7 InsightIDR
    """

    SIEM_SENTINEL = "siem_sentinel"
    """
    Microsoft Sentinel
    """

    SIEM_SPLUNK = "siem_splunk"
    """
    Splunk Enterprise Security
    """

    SIEM_SUMO_LOGIC = "siem_sumo_logic"
    """
    Sumo Logic Cloud SIEM
    """

    SINK_AWS_SECURITY_LAKE = "sink_aws_security_lake"
    """
    Amazon Security Lake
    """

    SINK_AWS_SQS = "sink_aws_sqs"
    """
    Amazon Simple Queue Service (SQS)
    """

    SINK_AZURE_MONITOR_LOGS = "sink_azure_monitor_logs"
    """
    Microsoft Azure Monitor Logs
    """

    SINK_CROWDSTRIKE_HEC = "sink_crowdstrike_hec"
    """
    CrowdStrike Falcon® Next-Gen SIEM (HEC)
    """

    SINK_ELASTICSEARCH = "sink_elasticsearch"
    """
    Elasticsearch
    """

    SINK_GOOGLE_SEC_OPS = "sink_google_sec_ops"
    """
    Google Security Operations (Chronicle Compatibility)
    """

    SINK_GOOGLE_SECURITY_OPERATIONS = "sink_google_security_operations"
    """
    Google Security Operations
    """

    SINK_MOCK = "sink_mock_sink"
    """
    Synqly Test Provider
    """

    SINK_OPEN_SEARCH = "sink_opensearch"
    """
    OpenSearch
    """

    SINK_SPLUNK = "sink_splunk"
    """
    Splunk Enterprise Security
    """

    STORAGE_AWS_S_3 = "storage_aws_s3"
    """
    Amazon S3
    """

    STORAGE_AZURE_BLOB = "storage_azure_blob"
    """
    Microsoft Azure Blob Storage
    """

    STORAGE_GCS = "storage_gcs"
    """
    Google Cloud Storage
    """

    STORAGE_MOCK = "storage_mock_storage"
    """
    Synqly Test Provider
    """

    TICKETING_AUTOTASK = "ticketing_autotask"
    """
    Autotask Operations Cloud
    """

    TICKETING_FRESHDESK = "ticketing_freshdesk"
    """
    Freshdesk
    """

    TICKETING_JIRA = "ticketing_jira"
    """
    Atlassian Jira
    """

    TICKETING_MOCK = "ticketing_mock_ticketing"
    """
    Synqly Test Provider
    """

    TICKETING_PAGER_DUTY = "ticketing_pagerduty"
    """
    PagerDuty Operations Cloud
    """

    TICKETING_SERVICE_NOW = "ticketing_servicenow"
    """
    ServiceNow IT Service Management (ITSM)
    """

    TICKETING_SERVICE_NOW_SIR = "ticketing_servicenow_sir"
    """
    ServiceNow Security Incident Response (SIR)
    """

    TICKETING_TORQ = "ticketing_torq"
    """
    Torq
    """

    TICKETING_ZENDESK = "ticketing_zendesk"
    """
    Zendesk
    """

    VULNERABILITIES_CROWD_STRIKE = "vulnerabilities_crowdstrike"
    """
    CrowdStrike Falcon® Spotlight
    """

    VULNERABILITIES_NUCLEUS = "vulnerabilities_nucleus"
    """
    Nucleus Vulnerability Management
    """

    VULNERABILITIES_QUALYS_CLOUD = "vulnerabilities_qualys_cloud"
    """
    Qualys Vulnerability Management, Detection & Response (VMDR)
    """

    VULNERABILITIES_QUALYS_CLOUD_MOCK = "vulnerabilities_qualys_cloud_mock"
    """
    [MOCK] Qualys Vulnerability Management, Detection & Response (VMDR)
    """

    VULNERABILITIES_RAPID_7_INSIGHT_CLOUD = "vulnerabilities_rapid7_insight_cloud"
    """
    Rapid7 InsightVM
    """

    VULNERABILITIES_RAPID_7_INSIGHT_CLOUD_MOCK = "vulnerabilities_rapid7_insight_cloud_mock"
    """
    [MOCK] Rapid7 Insight Vulnerability Management Cloud
    """

    VULNERABILITIES_TANIUM_CLOUD = "vulnerabilities_tanium_cloud"
    """
    Tanium Vulnerability Management
    """

    VULNERABILITIES_TANIUM_CLOUD_MOCK = "vulnerabilities_tanium_cloud_mock"
    """
    [MOCK] Tsanium Vulnerability Management
    """

    VULNERABILITIES_TENABLE_CLOUD = "vulnerabilities_tenable_cloud"
    """
    Tenable Vulnerability Management
    """

    ALL = "*"
    """
    Any provider config type.
    """

    def visit(
        self,
        assets_armis_centrix: typing.Callable[[], T_Result],
        assets_armis_centrix_mock: typing.Callable[[], T_Result],
        assets_axonius: typing.Callable[[], T_Result],
        assets_crowd_strike: typing.Callable[[], T_Result],
        assets_nozomi_vantage: typing.Callable[[], T_Result],
        assets_nozomi_vantage_mock: typing.Callable[[], T_Result],
        assets_qualys_cloud: typing.Callable[[], T_Result],
        assets_qualys_cloud_mock: typing.Callable[[], T_Result],
        assets_service_now: typing.Callable[[], T_Result],
        assets_service_now_mock: typing.Callable[[], T_Result],
        assets_sevco: typing.Callable[[], T_Result],
        assets_tanium_cloud: typing.Callable[[], T_Result],
        assets_tanium_cloud_mock: typing.Callable[[], T_Result],
        cloud_security_crowd_strike: typing.Callable[[], T_Result],
        cloud_security_defender: typing.Callable[[], T_Result],
        edr_crowd_strike: typing.Callable[[], T_Result],
        edr_defender: typing.Callable[[], T_Result],
        edr_malwarebytes: typing.Callable[[], T_Result],
        edr_sentinel_one: typing.Callable[[], T_Result],
        edr_sophos: typing.Callable[[], T_Result],
        edr_tanium: typing.Callable[[], T_Result],
        identity_entra_id: typing.Callable[[], T_Result],
        identity_google: typing.Callable[[], T_Result],
        identity_okta: typing.Callable[[], T_Result],
        identity_ping_one: typing.Callable[[], T_Result],
        notifications_jira: typing.Callable[[], T_Result],
        notifications_mock: typing.Callable[[], T_Result],
        notifications_slack_webhook: typing.Callable[[], T_Result],
        notifications_slack: typing.Callable[[], T_Result],
        notifications_teams: typing.Callable[[], T_Result],
        siem_crowdstrike: typing.Callable[[], T_Result],
        siem_elasticsearch: typing.Callable[[], T_Result],
        siem_google_chronicle: typing.Callable[[], T_Result],
        siem_google_security_operations: typing.Callable[[], T_Result],
        siem_mock: typing.Callable[[], T_Result],
        siem_open_search: typing.Callable[[], T_Result],
        siem_q_radar: typing.Callable[[], T_Result],
        siem_rapid_7_insight_idr: typing.Callable[[], T_Result],
        siem_sentinel: typing.Callable[[], T_Result],
        siem_splunk: typing.Callable[[], T_Result],
        siem_sumo_logic: typing.Callable[[], T_Result],
        sink_aws_security_lake: typing.Callable[[], T_Result],
        sink_aws_sqs: typing.Callable[[], T_Result],
        sink_azure_monitor_logs: typing.Callable[[], T_Result],
        sink_crowdstrike_hec: typing.Callable[[], T_Result],
        sink_elasticsearch: typing.Callable[[], T_Result],
        sink_google_sec_ops: typing.Callable[[], T_Result],
        sink_google_security_operations: typing.Callable[[], T_Result],
        sink_mock: typing.Callable[[], T_Result],
        sink_open_search: typing.Callable[[], T_Result],
        sink_splunk: typing.Callable[[], T_Result],
        storage_aws_s_3: typing.Callable[[], T_Result],
        storage_azure_blob: typing.Callable[[], T_Result],
        storage_gcs: typing.Callable[[], T_Result],
        storage_mock: typing.Callable[[], T_Result],
        ticketing_autotask: typing.Callable[[], T_Result],
        ticketing_freshdesk: typing.Callable[[], T_Result],
        ticketing_jira: typing.Callable[[], T_Result],
        ticketing_mock: typing.Callable[[], T_Result],
        ticketing_pager_duty: typing.Callable[[], T_Result],
        ticketing_service_now: typing.Callable[[], T_Result],
        ticketing_service_now_sir: typing.Callable[[], T_Result],
        ticketing_torq: typing.Callable[[], T_Result],
        ticketing_zendesk: typing.Callable[[], T_Result],
        vulnerabilities_crowd_strike: typing.Callable[[], T_Result],
        vulnerabilities_nucleus: typing.Callable[[], T_Result],
        vulnerabilities_qualys_cloud: typing.Callable[[], T_Result],
        vulnerabilities_qualys_cloud_mock: typing.Callable[[], T_Result],
        vulnerabilities_rapid_7_insight_cloud: typing.Callable[[], T_Result],
        vulnerabilities_rapid_7_insight_cloud_mock: typing.Callable[[], T_Result],
        vulnerabilities_tanium_cloud: typing.Callable[[], T_Result],
        vulnerabilities_tanium_cloud_mock: typing.Callable[[], T_Result],
        vulnerabilities_tenable_cloud: typing.Callable[[], T_Result],
        all_: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ProviderConfigId.ASSETS_ARMIS_CENTRIX:
            return assets_armis_centrix()
        if self is ProviderConfigId.ASSETS_ARMIS_CENTRIX_MOCK:
            return assets_armis_centrix_mock()
        if self is ProviderConfigId.ASSETS_AXONIUS:
            return assets_axonius()
        if self is ProviderConfigId.ASSETS_CROWD_STRIKE:
            return assets_crowd_strike()
        if self is ProviderConfigId.ASSETS_NOZOMI_VANTAGE:
            return assets_nozomi_vantage()
        if self is ProviderConfigId.ASSETS_NOZOMI_VANTAGE_MOCK:
            return assets_nozomi_vantage_mock()
        if self is ProviderConfigId.ASSETS_QUALYS_CLOUD:
            return assets_qualys_cloud()
        if self is ProviderConfigId.ASSETS_QUALYS_CLOUD_MOCK:
            return assets_qualys_cloud_mock()
        if self is ProviderConfigId.ASSETS_SERVICE_NOW:
            return assets_service_now()
        if self is ProviderConfigId.ASSETS_SERVICE_NOW_MOCK:
            return assets_service_now_mock()
        if self is ProviderConfigId.ASSETS_SEVCO:
            return assets_sevco()
        if self is ProviderConfigId.ASSETS_TANIUM_CLOUD:
            return assets_tanium_cloud()
        if self is ProviderConfigId.ASSETS_TANIUM_CLOUD_MOCK:
            return assets_tanium_cloud_mock()
        if self is ProviderConfigId.CLOUD_SECURITY_CROWD_STRIKE:
            return cloud_security_crowd_strike()
        if self is ProviderConfigId.CLOUD_SECURITY_DEFENDER:
            return cloud_security_defender()
        if self is ProviderConfigId.EDR_CROWD_STRIKE:
            return edr_crowd_strike()
        if self is ProviderConfigId.EDR_DEFENDER:
            return edr_defender()
        if self is ProviderConfigId.EDR_MALWAREBYTES:
            return edr_malwarebytes()
        if self is ProviderConfigId.EDR_SENTINEL_ONE:
            return edr_sentinel_one()
        if self is ProviderConfigId.EDR_SOPHOS:
            return edr_sophos()
        if self is ProviderConfigId.EDR_TANIUM:
            return edr_tanium()
        if self is ProviderConfigId.IDENTITY_ENTRA_ID:
            return identity_entra_id()
        if self is ProviderConfigId.IDENTITY_GOOGLE:
            return identity_google()
        if self is ProviderConfigId.IDENTITY_OKTA:
            return identity_okta()
        if self is ProviderConfigId.IDENTITY_PING_ONE:
            return identity_ping_one()
        if self is ProviderConfigId.NOTIFICATIONS_JIRA:
            return notifications_jira()
        if self is ProviderConfigId.NOTIFICATIONS_MOCK:
            return notifications_mock()
        if self is ProviderConfigId.NOTIFICATIONS_SLACK_WEBHOOK:
            return notifications_slack_webhook()
        if self is ProviderConfigId.NOTIFICATIONS_SLACK:
            return notifications_slack()
        if self is ProviderConfigId.NOTIFICATIONS_TEAMS:
            return notifications_teams()
        if self is ProviderConfigId.SIEM_CROWDSTRIKE:
            return siem_crowdstrike()
        if self is ProviderConfigId.SIEM_ELASTICSEARCH:
            return siem_elasticsearch()
        if self is ProviderConfigId.SIEM_GOOGLE_CHRONICLE:
            return siem_google_chronicle()
        if self is ProviderConfigId.SIEM_GOOGLE_SECURITY_OPERATIONS:
            return siem_google_security_operations()
        if self is ProviderConfigId.SIEM_MOCK:
            return siem_mock()
        if self is ProviderConfigId.SIEM_OPEN_SEARCH:
            return siem_open_search()
        if self is ProviderConfigId.SIEM_Q_RADAR:
            return siem_q_radar()
        if self is ProviderConfigId.SIEM_RAPID_7_INSIGHT_IDR:
            return siem_rapid_7_insight_idr()
        if self is ProviderConfigId.SIEM_SENTINEL:
            return siem_sentinel()
        if self is ProviderConfigId.SIEM_SPLUNK:
            return siem_splunk()
        if self is ProviderConfigId.SIEM_SUMO_LOGIC:
            return siem_sumo_logic()
        if self is ProviderConfigId.SINK_AWS_SECURITY_LAKE:
            return sink_aws_security_lake()
        if self is ProviderConfigId.SINK_AWS_SQS:
            return sink_aws_sqs()
        if self is ProviderConfigId.SINK_AZURE_MONITOR_LOGS:
            return sink_azure_monitor_logs()
        if self is ProviderConfigId.SINK_CROWDSTRIKE_HEC:
            return sink_crowdstrike_hec()
        if self is ProviderConfigId.SINK_ELASTICSEARCH:
            return sink_elasticsearch()
        if self is ProviderConfigId.SINK_GOOGLE_SEC_OPS:
            return sink_google_sec_ops()
        if self is ProviderConfigId.SINK_GOOGLE_SECURITY_OPERATIONS:
            return sink_google_security_operations()
        if self is ProviderConfigId.SINK_MOCK:
            return sink_mock()
        if self is ProviderConfigId.SINK_OPEN_SEARCH:
            return sink_open_search()
        if self is ProviderConfigId.SINK_SPLUNK:
            return sink_splunk()
        if self is ProviderConfigId.STORAGE_AWS_S_3:
            return storage_aws_s_3()
        if self is ProviderConfigId.STORAGE_AZURE_BLOB:
            return storage_azure_blob()
        if self is ProviderConfigId.STORAGE_GCS:
            return storage_gcs()
        if self is ProviderConfigId.STORAGE_MOCK:
            return storage_mock()
        if self is ProviderConfigId.TICKETING_AUTOTASK:
            return ticketing_autotask()
        if self is ProviderConfigId.TICKETING_FRESHDESK:
            return ticketing_freshdesk()
        if self is ProviderConfigId.TICKETING_JIRA:
            return ticketing_jira()
        if self is ProviderConfigId.TICKETING_MOCK:
            return ticketing_mock()
        if self is ProviderConfigId.TICKETING_PAGER_DUTY:
            return ticketing_pager_duty()
        if self is ProviderConfigId.TICKETING_SERVICE_NOW:
            return ticketing_service_now()
        if self is ProviderConfigId.TICKETING_SERVICE_NOW_SIR:
            return ticketing_service_now_sir()
        if self is ProviderConfigId.TICKETING_TORQ:
            return ticketing_torq()
        if self is ProviderConfigId.TICKETING_ZENDESK:
            return ticketing_zendesk()
        if self is ProviderConfigId.VULNERABILITIES_CROWD_STRIKE:
            return vulnerabilities_crowd_strike()
        if self is ProviderConfigId.VULNERABILITIES_NUCLEUS:
            return vulnerabilities_nucleus()
        if self is ProviderConfigId.VULNERABILITIES_QUALYS_CLOUD:
            return vulnerabilities_qualys_cloud()
        if self is ProviderConfigId.VULNERABILITIES_QUALYS_CLOUD_MOCK:
            return vulnerabilities_qualys_cloud_mock()
        if self is ProviderConfigId.VULNERABILITIES_RAPID_7_INSIGHT_CLOUD:
            return vulnerabilities_rapid_7_insight_cloud()
        if self is ProviderConfigId.VULNERABILITIES_RAPID_7_INSIGHT_CLOUD_MOCK:
            return vulnerabilities_rapid_7_insight_cloud_mock()
        if self is ProviderConfigId.VULNERABILITIES_TANIUM_CLOUD:
            return vulnerabilities_tanium_cloud()
        if self is ProviderConfigId.VULNERABILITIES_TANIUM_CLOUD_MOCK:
            return vulnerabilities_tanium_cloud_mock()
        if self is ProviderConfigId.VULNERABILITIES_TENABLE_CLOUD:
            return vulnerabilities_tenable_cloud()
        if self is ProviderConfigId.ALL:
            return all_()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Priority(str, enum.Enum):
    URGENT = "URGENT"
    CRITICAL = "CRITICAL"
    HIGH = "HIGH"
    MEDIUM = "MEDIUM"
    MODERATE = "MODERATE"
    LOW = "LOW"
    PLANNING = "PLANNING"

    def visit(
        self,
        urgent: typing.Callable[[], T_Result],
        critical: typing.Callable[[], T_Result],
        high: typing.Callable[[], T_Result],
        medium: typing.Callable[[], T_Result],
        moderate: typing.Callable[[], T_Result],
        low: typing.Callable[[], T_Result],
        planning: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is Priority.URGENT:
            return urgent()
        if self is Priority.CRITICAL:
            return critical()
        if self is Priority.HIGH:
            return high()
        if self is Priority.MEDIUM:
            return medium()
        if self is Priority.MODERATE:
            return moderate()
        if self is Priority.LOW:
            return low()
        if self is Priority.PLANNING:
            return planning()



# This file was auto-generated by Fern from our API Definition.

from . import (
    accountchange,
    apiactivity,
    applicationlifecycle,
    authentication,
    authorizesession,
    baseevent,
    compliancefinding,
    configstate,
    detectionfinding,
    dnsactivity,
    entitymanagement,
    fileactivity,
    groupmanagement,
    incidentfinding,
    inventoryinfo,
    moduleactivity,
    networkactivity,
    processactivity,
    scanactivity,
    scheduledjobactivity,
    securityfinding,
    softwareinfo,
    useraccessmanagement,
    vulnerabilityfinding,
    webresourceaccessactivity,
    webresourcesactivity,
)

__all__ = [
    "accountchange",
    "apiactivity",
    "applicationlifecycle",
    "authentication",
    "authorizesession",
    "baseevent",
    "compliancefinding",
    "configstate",
    "detectionfinding",
    "dnsactivity",
    "entitymanagement",
    "fileactivity",
    "groupmanagement",
    "incidentfinding",
    "inventoryinfo",
    "moduleactivity",
    "networkactivity",
    "processactivity",
    "scanactivity",
    "scheduledjobactivity",
    "securityfinding",
    "softwareinfo",
    "useraccessmanagement",
    "vulnerabilityfinding",
    "webresourceaccessactivity",
    "webresourcesactivity",
]



# This file was auto-generated by Fern from our API Definition.

from .actor import Actor, Actor_Agent, Actor_AgentSummary, Actor_GroupSummary, Actor_ScheduleSummary
from .agent import Agent
from .agent_summary import AgentSummary
from .attachment import Attachment
from .attachment_id import AttachmentId
from .attachment_metadata import AttachmentMetadata
from .comment import Comment
from .comment_id import CommentId
from .create_attachment_request import CreateAttachmentRequest
from .create_attachment_response import CreateAttachmentResponse
from .create_comment_request import CreateCommentRequest
from .create_comment_response import CreateCommentResponse
from .create_note_request import CreateNoteRequest
from .create_note_response import CreateNoteResponse
from .create_ticket_request import CreateTicketRequest
from .create_ticket_response import CreateTicketResponse
from .download_attachment_response import DownloadAttachmentResponse
from .escalation_policy import EscalationPolicy
from .escalation_policy_rule import EscalationPolicyRule
from .field_mapping_id import FieldMappingId
from .get_ticket_response import GetTicketResponse
from .group_summary import GroupSummary
from .issue_type_id import IssueTypeId
from .list_attachments_metadata_response import ListAttachmentsMetadataResponse
from .list_comments_response import ListCommentsResponse
from .list_notes_response import ListNotesResponse
from .list_on_call_response import ListOnCallResponse
from .list_projects_response import ListProjectsResponse
from .list_remote_fields_response import ListRemoteFieldsResponse
from .note import Note
from .note_id import NoteId
from .patch_note_response import PatchNoteResponse
from .patch_ticket_response import PatchTicketResponse
from .priority import Priority
from .project import Project
from .project_id import ProjectId
from .query_escalation_policies_response import QueryEscalationPoliciesResponse
from .query_tickets_response import QueryTicketsResponse
from .remote_field import RemoteField
from .remote_field_schema import RemoteFieldSchema
from .remote_field_scope import RemoteFieldScope
from .remote_field_type_id import RemoteFieldTypeId
from .schedule_summary import ScheduleSummary
from .status import Status
from .ticket import Ticket
from .ticket_id import TicketId
from .value_mapping_id import ValueMappingId

__all__ = [
    "Actor",
    "Actor_Agent",
    "Actor_AgentSummary",
    "Actor_GroupSummary",
    "Actor_ScheduleSummary",
    "Agent",
    "AgentSummary",
    "Attachment",
    "AttachmentId",
    "AttachmentMetadata",
    "Comment",
    "CommentId",
    "CreateAttachmentRequest",
    "CreateAttachmentResponse",
    "CreateCommentRequest",
    "CreateCommentResponse",
    "CreateNoteRequest",
    "CreateNoteResponse",
    "CreateTicketRequest",
    "CreateTicketResponse",
    "DownloadAttachmentResponse",
    "EscalationPolicy",
    "EscalationPolicyRule",
    "FieldMappingId",
    "GetTicketResponse",
    "GroupSummary",
    "IssueTypeId",
    "ListAttachmentsMetadataResponse",
    "ListCommentsResponse",
    "ListNotesResponse",
    "ListOnCallResponse",
    "ListProjectsResponse",
    "ListRemoteFieldsResponse",
    "Note",
    "NoteId",
    "PatchNoteResponse",
    "PatchTicketResponse",
    "Priority",
    "Project",
    "ProjectId",
    "QueryEscalationPoliciesResponse",
    "QueryTicketsResponse",
    "RemoteField",
    "RemoteFieldSchema",
    "RemoteFieldScope",
    "RemoteFieldTypeId",
    "ScheduleSummary",
    "Status",
    "Ticket",
    "TicketId",
    "ValueMappingId",
]



# This file was auto-generated by Fern from our API Definition.

from .....v_1_3_0.resources.moduleactivity.resources.classes.types.module_activity import (
    ModuleActivity as resources_ocsf_resources_v_1_3_0_resources_moduleactivity_resources_classes_types_module_activity_ModuleActivity,
)

ModuleActivity = (
    resources_ocsf_resources_v_1_3_0_resources_moduleactivity_resources_classes_types_module_activity_ModuleActivity
)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AsyncOperationRequestId,
    Operation,
    OperationError,
    OperationInput,
    OperationSchedule,
    OperationStatus,
)

__all__ = [
    "AsyncOperationRequestId",
    "Operation",
    "OperationError",
    "OperationInput",
    "OperationSchedule",
    "OperationStatus",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AdditionalMappingType(str, enum.Enum):
    RECOMMENDED = "recommended"
    OPTIONAL = "optional"
    FIXED = "fixed"

    def visit(
        self,
        recommended: typing.Callable[[], T_Result],
        optional: typing.Callable[[], T_Result],
        fixed: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is AdditionalMappingType.RECOMMENDED:
            return recommended()
        if self is AdditionalMappingType.OPTIONAL:
            return optional()
        if self is AdditionalMappingType.FIXED:
            return fixed()



# This file was auto-generated by Fern from our API Definition.

from .create_integration_web_hook_request import CreateIntegrationWebHookRequest
from .create_integration_web_hook_response import CreateIntegrationWebHookResponse
from .integration_web_hook import IntegrationWebHook
from .integration_web_hook_id import IntegrationWebHookId
from .list_integration_web_hooks_response import ListIntegrationWebHooksResponse

__all__ = [
    "CreateIntegrationWebHookRequest",
    "CreateIntegrationWebHookResponse",
    "IntegrationWebHook",
    "IntegrationWebHookId",
    "ListIntegrationWebHooksResponse",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class IdentityClassOv(str, enum.Enum):
    INDIVIDUAL = "individual"
    GROUP = "group"
    SYSTEM = "system"
    ORGANIZATION = "organization"
    CLASS = "class"
    UNKNOWN = "unknown"

    def visit(
        self,
        individual: typing.Callable[[], T_Result],
        group: typing.Callable[[], T_Result],
        system: typing.Callable[[], T_Result],
        organization: typing.Callable[[], T_Result],
        class_: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is IdentityClassOv.INDIVIDUAL:
            return individual()
        if self is IdentityClassOv.GROUP:
            return group()
        if self is IdentityClassOv.SYSTEM:
            return system()
        if self is IdentityClassOv.ORGANIZATION:
            return organization()
        if self is IdentityClassOv.CLASS:
            return class_()
        if self is IdentityClassOv.UNKNOWN:
            return unknown()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.url_string import UrlString
from .feature import Feature

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Product(pydantic.BaseModel):
    """
    The Product object describes characteristics of a software product.
    """

    cpe_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Common Platform Enumeration (CPE) name as described by (<a target='_blank' href='https://nvd.nist.gov/products/cpe'>NIST</a>) For example: <code>cpe:/a:apple:safari:16.2</code>.
    """

    feature: typing.Optional[Feature] = pydantic.Field(default=None)
    """
    The feature that reported the event.
    """

    lang: typing.Optional[str] = pydantic.Field(default=None)
    """
    The two letter lower case language codes, as defined by <a target='_blank' href='https://en.wikipedia.org/wiki/ISO_639-1'>ISO 639-1</a>. For example: <code>en</code> (English), <code>de</code> (German), or <code>fr</code> (French).
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the product.
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The installation path of the product.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the product.
    """

    url_string: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The URL pointing towards the product.
    """

    vendor_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the vendor of the product.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the product, as defined by the event source. For example: <code>2013.1.3-beta</code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .accounts_permissions import AccountsPermissions
from .alarm_policies_permissions import AlarmPoliciesPermissions
from .alarms_permissions import AlarmsPermissions
from .audit_permissions import AuditPermissions
from .auth_permissions import AuthPermissions
from .billing_permissions import BillingPermissions
from .bridges_permissions import BridgesPermissions
from .credentials_permissions import CredentialsPermissions
from .integration_points_permissions import IntegrationPointsPermissions
from .integrations_permissions import IntegrationsPermissions
from .mappings_permissions import MappingsPermissions
from .members_permissions import MembersPermissions
from .operations_permissions import OperationsPermissions
from .organization_permissions import OrganizationPermissions
from .permission_set_permissions import PermissionSetPermissions
from .read_write_permissions import ReadWritePermissions
from .roles_permissions import RolesPermissions
from .status_permissions import StatusPermissions
from .sub_orgs_permissions import SubOrgsPermissions
from .tokens_permissions import TokensPermissions
from .transforms_permissions import TransformsPermissions
from .webhooks_permissions import WebhooksPermissions

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ApiPermissionMap(pydantic.BaseModel):
    all_: typing.Optional[ReadWritePermissions] = pydantic.Field(alias="all", default=None)
    accounts: typing.Optional[AccountsPermissions] = None
    alarms: typing.Optional[AlarmsPermissions] = None
    alarm_policies: typing.Optional[AlarmPoliciesPermissions] = None
    audit: typing.Optional[AuditPermissions] = None
    auth: typing.Optional[AuthPermissions] = None
    billing: typing.Optional[BillingPermissions] = None
    bridges: typing.Optional[BridgesPermissions] = None
    credentials: typing.Optional[CredentialsPermissions] = None
    integrations: typing.Optional[IntegrationsPermissions] = None
    integration_points: typing.Optional[IntegrationPointsPermissions] = None
    operations: typing.Optional[OperationsPermissions] = None
    mappings: typing.Optional[MappingsPermissions] = None
    members: typing.Optional[MembersPermissions] = None
    organizations: typing.Optional[OrganizationPermissions] = None
    permission_set: typing.Optional[PermissionSetPermissions] = None
    roles: typing.Optional[RolesPermissions] = None
    status: typing.Optional[StatusPermissions] = None
    suborgs: typing.Optional[SubOrgsPermissions] = None
    tokens: typing.Optional[TokensPermissions] = None
    transforms: typing.Optional[TransformsPermissions] = None
    webhooks: typing.Optional[WebhooksPermissions] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .create_device_request import CreateDeviceRequest
from .create_device_request_generic import CreateDeviceRequestGeneric
from .create_device_response import CreateDeviceResponse
from .create_device_response_generic import CreateDeviceResponseGeneric
from .device import Device
from .get_labels_response import GetLabelsResponse
from .get_labels_response_generic import GetLabelsResponseGeneric
from .label import Label
from .query_devices_response import QueryDevicesResponse
from .query_devices_response_generic import QueryDevicesResponseGeneric

__all__ = [
    "CreateDeviceRequest",
    "CreateDeviceRequestGeneric",
    "CreateDeviceResponse",
    "CreateDeviceResponseGeneric",
    "Device",
    "GetLabelsResponse",
    "GetLabelsResponseGeneric",
    "Label",
    "QueryDevicesResponse",
    "QueryDevicesResponseGeneric",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.email_address import EmailAddress
from .domain_contact_type_id import DomainContactTypeId
from .location import Location

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class DomainContact(pydantic.BaseModel):
    """
    The contact information related to a domain registration, e.g., registrant, administrator, abuse, billing, or technical contact.
    """

    email_addr: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The user's primary email address.
    """

    location: typing.Optional[Location] = pydantic.Field(default=None)
    """
    Location details for the contract such as the city, state/province, country, etc.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The individual or organization name for the contact.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The number associated with the phone.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Domain Contact type, normalized to the caption of the <code>type_id</code> value. In the case of 'Other', it is defined by the source
    """

    type_id: DomainContactTypeId = pydantic.Field()
    """
    The normalized domain contact type ID.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the contact information, typically provided in WHOIS information.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .account import Account
from .organization import Organization

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Cloud(pydantic.BaseModel):
    """
    The Cloud object contains information about a cloud or Software-as-a-Service account or similar construct, such as AWS Account ID, regions, organizations, folders, compartments, tenants, etc.
    """

    account: typing.Optional[Account] = pydantic.Field(default=None)
    """
    The account object describes details about the account that was the source or target of the activity.
    """

    cloud_partition: typing.Optional[str] = pydantic.Field(default=None)
    """
    The canonical cloud partition name to which the region is assigned (e.g. AWS Partitions: aws, aws-cn, aws-us-gov).
    """

    org: typing.Optional[Organization] = pydantic.Field(default=None)
    """
    Organization and org unit relevant to the event or object.
    """

    project_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of a Cloud project.
    """

    provider: str = pydantic.Field()
    """
    The unique name of the Cloud services provider, such as AWS, MS Azure, GCP, etc.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the cloud region, as defined by the cloud provider.
    """

    zone: typing.Optional[str] = pydantic.Field(default=None)
    """
    The availability zone in the cloud region, as defined by the cloud provider.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ProviderEnvironment(str, enum.Enum):
    PRODUCTION = "prod"
    """
    Provider may be used with production accounts.
    """

    TEST = "test"
    """
    Provider may be used with test accounts.
    """

    def visit(self, production: typing.Callable[[], T_Result], test: typing.Callable[[], T_Result]) -> T_Result:
        if self is ProviderEnvironment.PRODUCTION:
            return production()
        if self is ProviderEnvironment.TEST:
            return test()
